=== modified file 'dev.cfg'
--- dev.cfg	2007-12-12 08:36:51 +0000
+++ dev.cfg	2007-01-14 05:40:40 +0000
@@ -36,10 +36,6 @@
 server.environment="production"
 #autoreload.package="loggerhead"
 
-# We do *not* want our query arguments turned into unicode strings (as
-# we use them for revision/file ids, and these must not be unicode).
-decoding_filter.on = False
-
 # session_filter.on = True
 
 # Set to True if you'd like to abort execution if a controller gets an

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-02-27 02:40:57 +0000
+++ loggerhead/history.py	2007-11-19 08:55:10 +0000
@@ -210,7 +210,10 @@
         self = cls()
         self._branch = branch
         self._last_revid = self._branch.last_revision()
-        self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
+        if self._last_revid is not None:
+            self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
+        else:
+            self._revision_graph = {}
 
         if name is None:
             name = self._branch.nick
@@ -220,20 +223,17 @@
         self._full_history = []
         self._revision_info = {}
         self._revno_revid = {}
-        if bzrlib.revision.is_null(self._last_revid):
-            self._merge_sort = []
-        else:
-            self._merge_sort = bzrlib.tsort.merge_sort(
-                self._revision_graph, self._last_revid, generate_revno=True)
+        self._merge_sort = bzrlib.tsort.merge_sort(self._revision_graph, self._last_revid, generate_revno=True)
         for (seq, revid, merge_depth, revno, end_of_merge) in self._merge_sort:
             self._full_history.append(revid)
             revno_str = '.'.join(str(n) for n in revno)
             self._revno_revid[revno_str] = revid
             self._revision_info[revid] = (seq, revid, merge_depth, revno_str, end_of_merge)
+
         # cache merge info
         self._where_merged = {}
         for revid in self._revision_graph.keys():
-            if self._revision_info[revid][2] == 0:
+            if not revid in self._full_history:
                 continue
             for parent in self._revision_graph[revid]:
                 self._where_merged.setdefault(parent, set()).add(revid)
@@ -244,21 +244,13 @@
     @classmethod
     def from_folder(cls, path, name=None):
         b = bzrlib.branch.Branch.open(path)
-        b.lock_read()
-        try:
-            return cls.from_branch(b, name)
-        finally:
-            b.unlock()
+        return cls.from_branch(b, name)
 
     @with_branch_lock
     def out_of_date(self):
         # the branch may have been upgraded on disk, in which case we're stale.
-        newly_opened = bzrlib.branch.Branch.open(self._branch.base)
         if self._branch.__class__ is not \
-               newly_opened.__class__:
-            return True
-        if self._branch.repository.__class__ is not \
-               newly_opened.repository.__class__:
+               bzrlib.branch.Branch.open(self._branch.base).__class__:
             return True
         return self._branch.last_revision() != self._last_revid
 
@@ -271,10 +263,6 @@
     def use_search_index(self, index):
         self._index = index
 
-    @property
-    def has_revisions(self):
-        return not bzrlib.revision.is_null(self.last_revid)
-
     @with_branch_lock
     def detach(self):
         # called when a new history object needs to be created, because the
@@ -657,13 +645,20 @@
     @with_branch_lock
     @with_bzrlib_read_lock
     def get_changes_uncached(self, revid_list):
-        revid_list = filter(lambda revid: not bzrlib.revision.is_null(revid),
-                            revid_list)
-        repo = self._branch.repository
-        rev_list = repo.get_revisions(
-            repo.get_graph().get_parent_map(revid_list))
+        # Because we may loop and call get_revisions multiple times (to throw
+        # out dud revids), we grab the bzrlib read lock.
+        while True:
+            try:
+                rev_list = self._branch.repository.get_revisions(revid_list)
+            except (KeyError, bzrlib.errors.NoSuchRevision), e:
+                # this sometimes happens with arch-converted branches.
+                # i don't know why. :(
+                self.log.debug('No such revision (skipping): %s', e)
+                revid_list.remove(e.revision)
+            else:
+                break
 
-        return [self._change_from_revision(rev) for rev in rev_list]
+        return [self._change_from_revision(rev) for rev in rev_list]        
 
     def _get_deltas_for_revisions_with_trees(self, revisions):
         """Produce a list of revision deltas.

=== modified file 'loggerhead/templates/browse.kid'
--- loggerhead/templates/browse.kid	2008-02-27 01:55:39 +0000
+++ loggerhead/templates/browse.kid	2007-06-08 11:52:33 +0000
@@ -34,7 +34,7 @@
             <tr>
             <td class="name"> <a href="${tg.url([ '/' + project.name, view.name ])}">${view.friendly_name}</a> </td>
             <td class="description"> ${view.description} </td>
-            <td class="last-update"> ${view.get_history().has_revisions and util.ago(view.last_updated()) or "(no revisions)"} </td>
+            <td class="last-update"> ${view.get_history().last_revid and util.ago(view.last_updated()) or "(no revisions)"} </td>
             <td class="feed">
                 <a href="${tg.url([ '/' + project.name, view.name, 'atom' ])}" title="atom feed">
                     <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" alt="atom feed" />

